@@ -28,6 +28,8 @@ import tempfile
 import time
 import multiprocessing
 
+from paunch import runner as containers_runner
+
 logger = None
 sh_script = '/var/lib/docker-puppet/docker-puppet.sh'
 container_cli = os.environ.get('CONTAINER_CLI', 'docker')
@@ -55,7 +57,8 @@ def get_logger():
 log = get_logger()
 log.info('Running docker-puppet')
 
-config_volume_prefix = os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data')
+config_volume_prefix = os.path.abspath(os.environ.get('CONFIG_VOLUME_PREFIX',
+                                                      '/var/lib/config-data'))
 log.debug('CONFIG_VOLUME_PREFIX: %s' % config_volume_prefix)
 if not os.path.exists(config_volume_prefix):
     os.makedirs(config_volume_prefix)
@@ -63,6 +66,9 @@ if not os.path.exists(config_volume_prefix):
 if container_cli == 'docker':
     cli_dcmd = ['--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro,z']
     env = {}
+    # FIXME: add log=log once we have paunch 4.0.1 in Pypi and promoted in RDO
+    RUNNER = containers_runner.DockerRunner(
+        'docker-puppet', cont_cmd='docker')
 elif container_cli == 'podman':
     # podman doesn't allow relabeling content in /usr and
     # doesn't support named volumes
@@ -70,6 +76,9 @@ elif container_cli == 'podman':
                 '--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro']
     # podman need to find dependent binaries that are in environment
     env = {'PATH': os.environ['PATH']}
+    # FIXME: add log=log once we have paunch 4.0.1 in Pypi and promoted in RDO
+    RUNNER = containers_runner.PodmanRunner(
+        'docker-puppet', cont_cmd='podman')
 else:
     log.error('Invalid container_cli: %s' % container_cli)
     sys.exit(1)
@@ -169,9 +178,9 @@ def rm_container(name):
 process_count = int(os.environ.get('PROCESS_COUNT',
                                    multiprocessing.cpu_count()))
 config_file = os.environ.get('CONFIG', '/var/lib/docker-puppet/docker-puppet.json')
+log.debug('CONFIG: %s' % config_file)
 # If specified, only this config_volume will be used
 config_volume_only = os.environ.get('CONFIG_VOLUME', None)
-log.debug('CONFIG: %s' % config_file)
 with open(config_file) as f:
     json_data = json.load(f)
 
@@ -300,19 +309,46 @@ with open(sh_script, 'w') as script_file:
                 rsync_srcs+=" $d"
             fi
         done
-        rsync -a -R --delay-updates --delete-after $rsync_srcs /var/lib/config-data/${NAME}
+        # On stack update, if a password was changed in a config file,
+        # some services (e.g. mysql) must change their internal state
+        # (e.g. password in mysql DB) when paunch restarts them; and
+        # they need the old password to achieve that.
+        # For those services, we update the config hash to notify
+        # paunch that a restart is needed, but we do not update the
+        # password file in docker-puppet and let the service
+        # regenerate it instead.
+        action=$(hiera -c /etc/puppet/hiera.yaml stack_action)
+        if [ "x$action" = "xUPDATE" ];then
+            password_files="/root/.my.cnf"
+        else
+            password_files=""
+        fi
+
+        exclude_files=""
+        for p in $password_files; do
+            exclude_files+=" --exclude=$p"
+        done
+        rsync -a -R --delay-updates --delete-after $exclude_files $rsync_srcs /var/lib/config-data/${NAME}
 
 
         # Also make a copy of files modified during puppet run
         # This is useful for debugging
         echo "Gathering files modified after $(stat -c '%y' $origin_of_time)"
         mkdir -p /var/lib/config-data/puppet-generated/${NAME}
-        rsync -a -R -0 --delay-updates --delete-after \
+        rsync -a -R -0 --delay-updates --delete-after $exclude_files \
                       --files-from=<(find $rsync_srcs -newer $origin_of_time -not -path '/etc/puppet*' -print0) \
                       / /var/lib/config-data/puppet-generated/${NAME}
 
         # Write a checksum of the config-data dir, this is used as a
         # salt to trigger container restart when the config changes
+        # note: while being excluded from the output, password files
+        # are still included in checksum computation
+        additional_checksum_files=""
+        for p in $password_files; do
+            if [ -f "$p" ]; then
+                additional_checksum_files+=" $p"
+            fi
+        done
         # We need to exclude the swift rings and their backup as those change over time and
         # containers do not need to restart if they change
         EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/swift/*.ring.gz'\ --exclude='*/etc/swift/*.builder'\ --exclude='*/etc/libvirt/passwd.db'
@@ -320,9 +356,9 @@ with open(sh_script, 'w') as script_file:
         # output because otherwise the sed command cannot work. The sed is
         # needed because puppet puts timestamps as comments in cron and
         # parsedfile resources, hence triggering a change at every redeploy
-        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/${NAME} | tar xO | \
+        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/${NAME} $additional_checksum_files | tar xO | \
                 sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/${NAME}.md5sum
-        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/puppet-generated/${NAME} --mtime='1970-01-01' | tar xO \
+        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/puppet-generated/${NAME} $additional_checksum_files --mtime='1970-01-01' | tar xO \
                 | sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/puppet-generated/${NAME}.md5sum
     fi
     """)
@@ -347,12 +383,14 @@ def mp_puppet_config(*args):
             man_file.write('include ::tripleo::packages\n')
             man_file.write(manifest)
 
-        rm_container('docker-puppet-%s' % config_volume)
+        uname = RUNNER.unique_container_name('docker-puppet-%s' %
+                                             config_volume)
+        rm_container(uname)
         pull_image(config_image)
 
         common_dcmd = [cli_cmd, 'run',
                 '--user', 'root',
-                '--name', 'docker-puppet-%s' % config_volume,
+                '--name', uname,
                 '--env', 'PUPPET_TAGS=%s' % puppet_tags,
                 '--env', 'NAME=%s' % config_volume,
                 '--env', 'HOSTNAME=%s' % short_hostname(),
@@ -367,7 +405,7 @@ def mp_puppet_config(*args):
                 '--volume', '/etc/pki/tls/certs/ca-bundle.crt:/etc/pki/tls/certs/ca-bundle.crt:ro',
                 '--volume', '/etc/pki/tls/certs/ca-bundle.trust.crt:/etc/pki/tls/certs/ca-bundle.trust.crt:ro',
                 '--volume', '/etc/pki/tls/cert.pem:/etc/pki/tls/cert.pem:ro',
-                '--volume', '%s:/var/lib/config-data/:rw,z' % os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data'),
+                '--volume', '%s:/var/lib/config-data/:rw' % config_volume_prefix,
                 # Syslog socket for puppet logs
                 '--volume', '/dev/log:/dev/log:rw',
                 # script injection
@@ -419,7 +457,7 @@ def mp_puppet_config(*args):
             if cmd_stderr:
                 log.debug(cmd_stderr)
             # only delete successful runs, for debugging
-            rm_container('docker-puppet-%s' % config_volume)
+            rm_container(uname)
 
         log.info('Finished processing puppet configs for %s' % (config_volume))
         return subproc.returncode
