@@ -125,6 +125,11 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
                                    'HeatApiLoggingSource': ['default'],
                                    'HeatEngineLoggingSource': ['default'],
                                    'KeystoneLoggingSource': ['default'],
+                                   'KeystoneErrorLoggingSource': ['default'],
+                                   'KeystoneAdminAccessLoggingSource': ['default'],
+                                   'KeystoneAdminErrorLoggingSource': ['default'],
+                                   'KeystoneMainAcccessLoggingSource': ['default'],
+                                   'KeystoneMainErrorLoggingSource': ['default'],
                                    'MongoDbLoggingSource': ['default'],
                                    'NeutronApiLoggingSource': ['default'],
                                    'NeutronDhcpAgentLoggingSource': ['default'],
@@ -1111,8 +1116,11 @@ def validate(filename, param_map):
         # NOTE(hjensas): The routed network data example is very different ...
         # We need to develop a more advanced validator, probably using a schema
         # definition instead.
+        # NOTE(mandre): Same goes for the openshift network data where it
+        # contains only a subset of the overcloud networks.
         if (filename.startswith('./network_data_') and
-                not filename.endswith('routed.yaml')):
+                not (filename.endswith('routed.yaml') or
+                     filename.endswith('openshift.yaml'))):
             result = validate_network_data_file(filename)
             retval |= result
         else:
